package model;

import utils.LayerListUtils;
import listener.LayerListener;

public class BambooModel {
	private LayerList layerList;
	private Coordinate coordinate;
	private SaveController saveController;

	public enum State {
		UNLOCK, HALF_LOCK, LOCK
	}

	private State state = State.UNLOCK;

	public BambooModel() {
		layerList = new LayerList(20, 20);
		coordinate = new Coordinate(layerList);
		saveController = new SaveController(coordinate, layerList);
	}

	public LayerList getLayerList() {
		return layerList;
	}

	public Coordinate getCoordinate() {
		return coordinate;
	}

	public SaveController getSaveController() {
		return saveController;
	}

	public void addLayerListener(LayerListener bambooPanel) {
		layerList.addLayerListener(bambooPanel);
	}

	public void reverse() {
		layerList.reverse();
	}

	public void rotate() {
		layerList.rotate();
	}

	public void setState(State state) {
		this.state = state;
	}

	public void setMap(int masu, int minX, int maxX, int minY, int maxY) {
		if (state == State.UNLOCK)
			LayerListUtils.setMap(getLayerList(), masu, minX, maxX, minY, maxY);
		else 
			if(masu == Cell.MASU)
				LayerListUtils.setMapLocked(getLayerList(), Cell.MASU, minX, maxX, minY, maxY);
	}

	/** ׂ͈͓ď */
	public void deleteRegion() {
		if (state == State.UNLOCK)
			LayerListUtils.deleteRegion(getLayerList(), coordinate);
	}

	/** V}̐ݒ */
	public void setData(int i) {
		if (state == State.UNLOCK) {
			LayerListUtils.setNumber(getLayerList(), i, coordinate);
		}
	}

	/** |̐ݒ */
	public void setBamboo(String string) {
		if (state == State.UNLOCK) {
			LayerListUtils.setBamboo(getLayerList(), string, coordinate.getX(),
					coordinate.getY());
		} else if (state == State.HALF_LOCK) {
			LayerListUtils.setBambooLocked(getLayerList(), string, coordinate
					.getX(), coordinate.getY());
		}
	}

	/** V}ł */
	public void makeIsland() {
		if (this.state == State.UNLOCK)
			LayerListUtils.makeIsland(getLayerList(), this.coordinate.getX(),
					this.coordinate.getY());
		else if (this.state == State.HALF_LOCK)
			LayerListUtils.makeIslandLocked(getLayerList(), this.coordinate.getX(),
					this.coordinate.getY());
	}

	/** oct */
	public void setCross() {
		if (state == State.UNLOCK)
			LayerListUtils.setCross(getLayerList(), coordinate.getX(),
					coordinate.getY());
		else if (state == State.HALF_LOCK)
			LayerListUtils.setCrossLocked(getLayerList(), coordinate.getX(),
					coordinate.getY());
	}

	public int getColumnCount() {
		return layerList.getColumnCount();
	}

	public int getRowCount() {
		return layerList.getRowCount();
	}

}
